﻿#region Using directives

using System;
using System.ServiceModel.Channels;
using System.Xml;
using BTS;
using Microsoft.BizTalk.Message.Interop;
using Microsoft.Samples.BizTalk.Adapter.Common;
using Microsoft.Samples.BizTalk.Adapters.Wcf.Utility;
using Microsoft.XLANGs.BaseTypes;

#endregion

namespace Microsoft.Samples.BizTalk.Adapters.Wcf.Configuration
{
    internal enum BindingProfileType
    {
        Custom,
        BasicProfile,
        WSProfile
    }

    internal abstract class WcfAdapterProperties : ConfigProperties
    {
        private bool _useNativeOneWay;
        private Uri _endpointUri;
        private string _endpointAction;
        private string _bindingConfigurationName;
        private Type _bindingType;
        private IdentityType _identityType;
        private string _identityValue;
        private string _behaviorConfiguration;

        private string _uri;
        private bool _isTwoWay;

        private static PropertyBase isSolicitResponseProp = new IsSolicitResponse();

        public WcfAdapterProperties(string url)
        {
            _uri = url;
            _useNativeOneWay = true;
            _bindingType = typeof (CustomBinding);
            _identityType = IdentityType.None;
            _identityValue = null;
            _behaviorConfiguration = null;
        }

        public WcfAdapterProperties(IBaseMessage msg, string propertyNamespace)
        {
            _useNativeOneWay = true;
            _bindingType = typeof (CustomBinding);
            _identityType = IdentityType.None;
            _identityValue = null;
            _behaviorConfiguration = null;

            SystemMessageContext smc = new SystemMessageContext(msg.Context);
            _uri = smc.InboundTransportLocation;

            XmlDocument locationConfigDom = null;

            //  get the adapter configuration off the message
            string config = (string) msg.Context.Read("AdapterConfig", propertyNamespace);
            _isTwoWay = (bool) msg.Context.Read(isSolicitResponseProp.Name.Name, isSolicitResponseProp.Name.Namespace);

            //  the config can be null all that means is that we are doing a dynamic send
            if (null != config)
            {
                locationConfigDom = new XmlDocument();
                locationConfigDom.LoadXml(config);

                //  For Dynamic Sends the Location config can be null
                //  Location properties - possibly override some handler properties
                LocationConfiguration(locationConfigDom);
            }
        }

        protected abstract void LocationConfiguration(XmlDocument config);

        public bool IsTwoWay
        {
            get { return _isTwoWay; }
            protected set { _isTwoWay = value; }
        }

        public string Uri
        {
            get { return _uri; }
            set { _uri = value; }
        }

        public Uri EndpointUri
        {
            get { return _endpointUri; }
        }

        public string EndpointAction
        {
            get { return _endpointAction; }
        }

        public bool UseNativeOneWay
        {
            get { return _useNativeOneWay; }
        }

        public Type BindingType
        {
            get { return _bindingType; }
        }

        public string BindingConfiguration
        {
            get { return _bindingConfigurationName; }
        }

        public IdentityType IdentityType
        {
            get { return _identityType; }
        }

        public string IdentityValue
        {
            get { return _identityValue; }
        }


        public string BehaviorConfiguration
        {
            get { return _behaviorConfiguration; }
        }

        protected void LoadCommonConfiguration(XmlDocument endpointConfig)
        {
            _endpointUri = new Uri(Extract(endpointConfig, ConfigNames.Address.Uri, null));
            _endpointAction = Extract(endpointConfig, ConfigNames.Contract.Action, null);
            _useNativeOneWay = IfExistsExtractBool(endpointConfig, ConfigNames.Contract.NativeOneWay, true);
            _bindingConfigurationName = IfExistsExtract(endpointConfig, ConfigNames.Binding.Configuration, null);
            _identityType = ParseOption<IdentityType>(Extract(endpointConfig, ConfigNames.Address.IdentityType, null));
            _identityValue = IfExistsExtract(endpointConfig, ConfigNames.Address.IdentityValue, null);
            _behaviorConfiguration = IfExistsExtract(endpointConfig, ConfigNames.Behavior.Configuration, null);
            _bindingType = Type.GetType(Extract(endpointConfig, ConfigNames.Binding.Type, null), true);
        }

        public static T ParseOption<T>(string optionName)
        {
            return (T) Enum.Parse(typeof (T), optionName);
        }
    }
}