using System;
using System.Threading;
using Microsoft.BizTalk.Message.Interop;
using Microsoft.BizTalk.TransportProxy.Interop;
using Microsoft.Samples.BizTalk.Adapter.Common;

namespace Microsoft.Samples.BizTalk.Adapters.Wcf.Receiver
{
    internal sealed class WcfRequestResponseHandler : IBTTransmitter
    {
        private IBTTransportProxy _transportProxy;
        private AutoResetEvent _requestCompleted;
        private AutoResetEvent _responseReady;
        private IBaseMessage _responseMessage;
        private bool _isDirty;
        private bool _requestSubmitted;
        private bool _requestSuccessful;
        private ControlledTermination _control;

        public WcfRequestResponseHandler(IBTTransportProxy transportProxy, ControlledTermination control)
        {
            _transportProxy = transportProxy;
            _responseReady = new AutoResetEvent(false);
            _requestCompleted = new AutoResetEvent(false);
            _control = control;
            _responseMessage = null;
            _isDirty = false;
            _requestSubmitted = false;
            _requestSuccessful = false;
        }

        public void Initialize(IBTTransportProxy proxy)
        {
        }

        public void Terminate()
        {
        }

        public bool TransmitMessage(IBaseMessage msg)
        {
            _responseMessage = msg;
            _responseReady.Set();

            // Manual Delete
            return false;
        }

        public IBaseMessage GetResponseMessage()
        {
            if (_requestSubmitted == false)
                throw new InvalidOperationException("Message not submitted.");

            // Wait for callback
            _requestCompleted.WaitOne();

            if (_requestSuccessful)
            {
                _responseReady.WaitOne();
                return _responseMessage;
            }
            else
            {
                return null;
            }
        }

        public void ResponseProcessed()
        {
            if (_responseMessage == null)
                throw new InvalidOperationException("There is no response to post-process.");

            ReceiveBatch th = new ReceiveBatch(_transportProxy, _control, (ReceiveBatchCompleteHandler) null, 1);
            th.DeleteMessage(_responseMessage, null);
            th.Done();
        }

        public void SubmitRequestMessage(IBaseMessage requestMsg, object userData)
        {
            if (_isDirty)
                throw new InvalidOperationException(@"This handler only handles a single request/response pair");

            _isDirty = true;

            ReceiveBatch batch = new ReceiveBatch(_transportProxy, _control, BatchComplete, 1);

            string correlationToken = Guid.NewGuid().ToString();
            batch.SubmitRequestMessage(requestMsg, correlationToken, true, new DateTime(0), this, userData);
            batch.Done(null);
            _requestSubmitted = true;
        }

        public void BatchComplete(bool overallSuccess)
        {
            _requestSuccessful = overallSuccess;

            _requestCompleted.Set();
        }
    }
}